#include <Wire.h>
#include <LiquidCrystal_I2C.h>
#include "DHT.h"
#define DHTPIN 5
#define DHTTYPE DHT11
#include <Keypad.h>

DHT dht(DHTPIN, DHTTYPE);
LiquidCrystal_I2C lcd(0x27, 16, 2);
int led = 6;
int j = 0;
int sensor = 3;
int val;
const byte ROWS = 4;
const byte COLS = 4;
char keys[ROWS][COLS] = {
  {'1','2','3','A'},
  {'4','5','6','B'},
  {'7','8','9','C'},
  {'*','0','#','D'}
};
byte rowPins[ROWS] = {7, 8, 9, 10};
byte colPins[COLS] = {11, 12, 13, 2};
String inputPassword = "";
String correctPassword = "1234"; // Set your desired password here
bool accessGranted = false;
bool inResetMode = false;

Keypad keypad = Keypad(makeKeymap(keys), rowPins, colPins, ROWS, COLS);

void setup() {
  lcd.init(); // initialize the lcd
  Serial.begin(9600);
  pinMode(led, OUTPUT);
  pinMode(sensor, INPUT);
  dht.begin();
  lcd.backlight();
}

void resetPassword() {
  lcd.clear();
  lcd.print("New:");
  String newPassword = "";
  char key;
  while(newPassword.length() < 4) {
    key = keypad.getKey();
    if (key) {
      newPassword += key;
      lcd.setCursor(6 + newPassword.length(), 0);
      lcd.print("*");
      delay(500);
    }
  }
  lcd.setCursor(0, 1);
  lcd.print("Confirm:");
  String confirmPass = "";
  while(confirmPass.length() < 4) {
    key = keypad.getKey();
    if (key) {
      confirmPass += key;
      lcd.setCursor(8 + confirmPass.length(), 1);
      lcd.print("*");
      delay(500);
    }
  }
  if (newPassword == confirmPass) {
    correctPassword = newPassword;
    lcd.clear();
    lcd.print("Password Set!");
  } else {
    lcd.clear();
    lcd.print("Mismatch!");
  }
  delay(2000);
  lcd.clear();
  inResetMode = false;
}

void enterPIN(){
  char key = keypad.getKey();
  if (key) {
    if (key == '*') {  // Check if reset is initiated
      inResetMode = true;
      resetPassword();
      lcd.clear();
      lcd.print("Enter PIN:");
      j = 0;
      return;
    }
    if (key == '#') {
      if (inputPassword == correctPassword) {
        lcd.clear();
        lcd.print("Access Granted");
        accessGranted = true;
        delay(2000);
      } else {
        lcd.clear();
        lcd.print("Access Denied");
        inputPassword = "";
        j=0;
        delay(2000);
        lcd.clear();
        lcd.print("Enter PIN:");
      }
    } else if (key != '*' && key != '#') {
      inputPassword += key;
      lcd.setCursor(10+j, 0);
      j++;
      lcd.print("*");
    }
  }
}

void loop() {
  val = digitalRead(sensor);
  if (val == HIGH) {
    digitalWrite(led, LOW);
    lcd.clear();
    lcd.noBacklight();
    accessGranted = false;  // Reset for next loop iteration
    inputPassword = "";  // Clear password
    j=0;
  } else {
    lcd.backlight();
    lcd.setCursor(1, 0);
    lcd.print("Welcome home!");
    delay(2000);
    lcd.clear();
    lcd.print("Enter PIN:");

    while (!accessGranted && !inResetMode) {
      enterPIN();
    }
 
    if (accessGranted) {
      float humidity = dht.readHumidity();
      float temperature = dht.readTemperature();

      digitalWrite(led, HIGH);
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.print("Temperature: ");
      lcd.print(temperature);
      lcd.setCursor(0, 1);
      lcd.print("Humidity: ");
      lcd.print(humidity);
      delay(4000);
      lcd.clear();
      inputPassword = ""; // Clear password after use
      j = 0; // Reset cursor position for next PIN entry
    }
  }
}
